"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isRejected = exports.isFulfilled = void 0;
exports.settleAll = settleAll;
/** narrow promise.allSettled results to the successes, with provided type */
const isFulfilled = (s) => s.status === 'fulfilled';
exports.isFulfilled = isFulfilled;
/** narrow promise.allSettled results to the faiures.  Result is untyped */
const isRejected = (s) => s.status === 'rejected';
exports.isRejected = isRejected;
/**
 * Wrapper for promise.allSettled that returns typed errors
 *
 * @param promises Array of promises to settle
 *
 * @example
 * ```
 * const promises = [Promise.resolve(1), Promise.resolve(2), Promise.resolve(3)];
 * const { fulfilled, rejected } = await settleAll<number, SfError>(promises);
 * ```
 */
async function settleAll(promises) {
    const allSettled = await Promise.allSettled(promises);
    return {
        fulfilled: allSettled.filter(exports.isFulfilled).map((s) => s.value),
        rejected: allSettled.filter(exports.isRejected).map((s) => s.reason),
    };
}
//# sourceMappingURL=settleAll.js.map