/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Messages } from '@salesforce/core';
import { Duration } from '@salesforce/kit';
import { Flags, SfCommand } from '@salesforce/sf-plugins-core';
import { baseFlags, runBulkOperation } from '../../../bulkOperationBase.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-data', 'bulkv2.delete');
export default class Delete extends SfCommand {
    static examples = messages.getMessages('examples');
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static flags = {
        ...baseFlags,
        'hard-delete': Flags.boolean({
            summary: messages.getMessage('flags.hard-delete.summary'),
            description: messages.getMessage('flags.hard-delete.description'),
            default: false,
        }),
    };
    async run() {
        const { flags } = await this.parse(Delete);
        return runBulkOperation({
            cmd: this,
            sobject: flags.sobject,
            csvFileName: flags.file,
            connection: flags['target-org'].getConnection(flags['api-version']),
            wait: flags.async ? Duration.minutes(0) : flags.wait,
            verbose: flags.verbose,
            operation: flags['hard-delete'] ? 'hardDelete' : 'delete',
        });
    }
}
//# sourceMappingURL=bulk.js.map