/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Messages } from '@salesforce/core';
import { SfCommand, Flags, Ux } from '@salesforce/sf-plugins-core';
import { orgFlags } from '../../../../flags.js';
import { ExportApi } from '../../../../api/data/tree/exportApi.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-data', 'tree.export');
export default class Export extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static hidden = true;
    static state = 'deprecated';
    static deprecationOptions = {
        to: 'data tree export',
        message: messages.getMessage('LegacyDeprecation'),
    };
    static flags = {
        ...orgFlags,
        query: Flags.string({
            char: 'q',
            summary: messages.getMessage('flags.query.summary'),
            required: true,
        }),
        plan: Flags.boolean({
            char: 'p',
            summary: messages.getMessage('flags.plan.summary'),
        }),
        prefix: Flags.string({
            char: 'x',
            summary: messages.getMessage('flags.prefix.summary'),
        }),
        'output-dir': Flags.directory({
            char: 'd',
            summary: messages.getMessage('flags.output-dir.summary'),
            aliases: ['outputdir'],
            deprecateAliases: true,
        }),
    };
    async run() {
        const { flags } = await this.parse(Export);
        const ux = new Ux({ jsonEnabled: this.jsonEnabled() });
        const exportApi = new ExportApi(flags['target-org'], ux);
        const exportConfig = {
            outputDir: flags['output-dir'],
            plan: flags.plan,
            prefix: flags.prefix,
            query: flags.query,
        };
        return exportApi.export(exportConfig);
    }
}
//# sourceMappingURL=tree.js.map