/*
 * Copyright (c) 2024, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { SfCommand, Flags } from '@salesforce/sf-plugins-core';
import { Messages } from '@salesforce/core';
import { bulkIngest, columnDelimiterFlag } from '../../../bulkIngest.js';
import { BulkUpdateRequestCache } from '../../../bulkDataRequestCache.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-data', 'data.update.bulk');
export default class DataUpdateBulk extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static flags = {
        async: Flags.boolean({
            summary: messages.getMessage('flags.async.summary'),
            char: 'a',
        }),
        wait: Flags.duration({
            summary: messages.getMessage('flags.wait.summary'),
            char: 'w',
            unit: 'minutes',
        }),
        file: Flags.file({
            summary: messages.getMessage('flags.file.summary'),
            char: 'f',
            required: true,
            exists: true,
        }),
        sobject: Flags.string({
            summary: messages.getMessage('flags.sobject.summary'),
            char: 's',
            required: true,
        }),
        'api-version': Flags.orgApiVersion(),
        'target-org': Flags.requiredOrg(),
        'line-ending': Flags.option({
            summary: messages.getMessage('flags.line-ending.summary'),
            dependsOn: ['file'],
            options: ['CRLF', 'LF'],
        })(),
        'column-delimiter': columnDelimiterFlag,
    };
    async run() {
        const { flags } = await this.parse(DataUpdateBulk);
        return bulkIngest({
            resumeCmdId: 'data update resume',
            stageTitle: 'Updating data',
            object: flags.sobject,
            operation: 'update',
            lineEnding: flags['line-ending'],
            columnDelimiter: flags['column-delimiter'],
            conn: flags['target-org'].getConnection(flags['api-version']),
            cache: await BulkUpdateRequestCache.create(),
            async: flags.async,
            wait: flags.wait,
            file: flags.file,
            jsonEnabled: this.jsonEnabled(),
            logFn: (...args) => {
                this.log(...args);
            },
        });
    }
}
//# sourceMappingURL=bulk.js.map