/*
 * Copyright (c) 2021, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { dirname } from 'node:path';
import { SfCommand, Flags } from '@salesforce/sf-plugins-core';
import { Messages } from '@salesforce/core/messages';
import select from '@inquirer/select';
import { apiNamePrompt } from '../../../shared/prompts/apiName.js';
import { pluralPrompt } from '../../../shared/prompts/plural.js';
import { directoryPrompt } from '../../../shared/prompts/directory.js';
import { descriptionPrompt } from '../../../shared/prompts/description.js';
import { writeObjectFile } from '../../../shared/fs.js';
import { labelValidation } from '../../../shared/flags.js';
import { toSelectOption } from '../../../shared/prompts/functions.js';
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-sobject', 'generate.event');
export default class PlatformEventGenerate extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static requiresProject = true;
    static enableJsonFlag = false;
    static aliases = ['generate:metadata:platformevent'];
    static deprecateAliases = true;
    static flags = {
        label: Flags.string({
            char: 'l',
            summary: messages.getMessage('flags.label.summary'),
            required: true,
            parse: labelValidation,
        }),
    };
    async run() {
        const { flags } = await this.parse(PlatformEventGenerate);
        const directory = await directoryPrompt(this.project.getPackageDirectories());
        const objectToWrite = {
            fullName: await apiNamePrompt(flags.label, 'PlatformEvent'),
            pluralLabel: await pluralPrompt(flags.label),
            description: await descriptionPrompt(),
            publishBehavior: await select({
                message: messages.getMessage('prompts.publishBehavior'),
                choices: ['PublishImmediately', 'PublishAfterCommit'].map(toSelectOption),
            }),
            deploymentStatus: 'Deployed',
            eventType: 'HighVolume',
            label: flags.label,
        };
        this.styledJSON(objectToWrite);
        const writePath = await writeObjectFile(directory, objectToWrite);
        this.info(messages.getMessage('success.field', [dirname(writePath)]));
        return { object: objectToWrite, path: writePath };
    }
}
//# sourceMappingURL=platformevent.js.map