/*
 * Copyright (c) 2019, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
import { Flags, loglevel, SfCommand, orgApiVersionFlagWithDeprecations, Ux } from '@salesforce/sf-plugins-core';
import { TemplateType } from '@salesforce/templates';
import { CreateUtil } from '@salesforce/templates/lib/utils/index.js';
import { Messages } from '@salesforce/core';
import { getCustomTemplates, runGenerator } from '../../../utils/templateCommand.js';
import { internalFlag, outputDirFlagLightning } from '../../../utils/flags.js';
const lightningTestFileSuffix = /.resource$/;
Messages.importMessagesDirectoryFromMetaUrl(import.meta.url);
const messages = Messages.loadMessages('@salesforce/plugin-templates', 'lightningTest');
const lightningMessages = Messages.loadMessages('@salesforce/plugin-templates', 'lightning');
export default class LightningTest extends SfCommand {
    static summary = messages.getMessage('summary');
    static description = messages.getMessage('description');
    static examples = messages.getMessages('examples');
    static aliases = ['force:lightning:test:create'];
    static deprecateAliases = true;
    static flags = {
        name: Flags.string({
            char: 'n',
            summary: lightningMessages.getMessage('flags.name.summary', ['Test']),
            description: messages.getMessage('flags.name.description'),
            required: true,
            aliases: ['testname'],
            deprecateAliases: true,
        }),
        template: Flags.string({
            char: 't',
            summary: lightningMessages.getMessage('flags.template.summary'),
            description: lightningMessages.getMessage('flags.template.description'),
            default: 'DefaultLightningTest',
            options: CreateUtil.getCommandTemplatesForFiletype(lightningTestFileSuffix, 'lightningtest'),
        }),
        'output-dir': outputDirFlagLightning,
        internal: internalFlag,
        'api-version': orgApiVersionFlagWithDeprecations,
        loglevel,
    };
    async run() {
        const { flags } = await this.parse(LightningTest);
        // translate the new flags to the old ones the generator expects
        const flagsAsOptions = {
            testname: flags.name,
            template: 'DefaultLightningTest',
            outputdir: flags['output-dir'],
            internal: flags.internal,
            apiversion: flags['api-version'],
        };
        return runGenerator({
            templateType: TemplateType.LightningTest,
            opts: flagsAsOptions,
            ux: new Ux({ jsonEnabled: this.jsonEnabled() }),
            templates: getCustomTemplates(this.configAggregator),
        });
    }
}
//# sourceMappingURL=test.js.map